import {Module} from '@nestjs/common';
import {JwtModule} from '@nestjs/jwt';
import {TypeOrmModule} from '@nestjs/typeorm';
import JwtConfigService from 'src/core/jwt/jwt-config.service';
import JwtHelper from 'src/core/jwt/jwt.helper';
import {MailSenderService} from 'src/modules/mail-sender/mail-sender.service';
import {PaymentConfigurationRepository} from 'src/modules/payment-configuration/payment-configuration.repository';
import {PaymentGatewayRepository} from 'src/modules/payment-gateway/payment-gateway.repository';
import {RefundItemRepository} from 'src/modules/refund-item/refund-item.repository';
import {ReturnProductsRepository} from 'src/modules/return-products/return-products.repository';
import {UpsConfigurationRepository} from 'src/modules/ups-configuration/ups-configuration.repository';
import {UpsCoreService} from 'src/modules/ups/ups-core/ups-core.service';
import {UpsShippingService} from 'src/modules/ups/ups-shipping/ups-shipping.service';
import {UsersRepository} from 'src/modules/users/user.repository';
import {CouponRepository} from '../coupon/coupon.repository';
import {OrderDetailsRepository} from '../order-details/order-details.repository';
import {OrderRepository} from './order-repository';
import {OrdersController} from './orders.controller';
import {OrdersService} from './orders.service';
import { OrderDetails } from '../order-details/entities/order-details.entity';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { DoSpacesServicerovider } from 'src/modules/SpacesModule/SpacesService';
import { ProductsRepository } from 'src/modules/product/product/product.repository';
import { ContactPageCmsRepository } from 'src/modules/CMS/contact-page-cms/contact-page-cms.repository';

@Module({
    imports: [
        JwtModule.registerAsync({
            useClass: JwtConfigService
        }),
        TypeOrmModule.forFeature([UsersRepository, PaymentGatewayRepository, ReturnProductsRepository]),
        TypeOrmModule.forFeature([OrderRepository, RefundItemRepository]),
        TypeOrmModule.forFeature([ProductsRepository, ContactPageCmsRepository]),
        TypeOrmModule.forFeature([CouponRepository, PaymentGatewayRepository]),
        TypeOrmModule.forFeature([OrderDetailsRepository, PaymentConfigurationRepository]),
        TypeOrmModule.forFeature([UsersRepository, UpsConfigurationRepository]),
        TypeOrmModule.forFeature([OrderDetails]),

    ],
    controllers: [OrdersController],
    providers: [OrdersService, JwtHelper, MailSenderService, UpsShippingService, UpsCoreService, DoSpacesService, DoSpacesServicerovider],
    exports:[OrdersService]
})
export class OrdersModule {
}
