import {Module} from '@nestjs/common';
import {JwtModule} from '@nestjs/jwt';
import JwtConfigService from 'src/core/jwt/jwt-config.service';
import JwtHelper from 'src/core/jwt/jwt.helper';
import {StripeController} from './stripe.controller';
import {StripeService} from './stripe.service';
import { StripeRepository } from './stripe.repository';
import { TypeOrmModule } from '@nestjs/typeorm';
import { StripeCheckoutRepository } from './stripe-checkout.repository';
import { UpsCoreModule } from 'src/modules/ups/ups-core/ups-core.module';
import { OrderRepository } from '../order/order-repository';
import { OrdersModule } from '../order/orders.module';



@Module({
    imports: [
        JwtModule.registerAsync({
            useClass: JwtConfigService
        }),
        TypeOrmModule.forFeature([
            StripeRepository, 
            StripeCheckoutRepository,
            OrderRepository
        ]),
        UpsCoreModule,
        OrdersModule,
    ],
    controllers: [StripeController],
    providers: [StripeService, JwtHelper],
    exports:[StripeService]
})
export class StripeModule {
}
