import { Module } from '@nestjs/common';
import { PaymentGatewayService } from './payment-gateway.service';
import { PaymentGatewayController } from './payment-gateway.controller';
import { TypeOrmModule } from '@nestjs/typeorm';
import { PaymentConfigurationRepository } from '../payment-configuration/payment-configuration.repository';
import { PaymentGatewayRepository } from './payment-gateway.repository';
import { JwtModule } from '@nestjs/jwt';
import JwtConfigService from 'src/core/jwt/jwt-config.service';
import JwtHelper from 'src/core/jwt/jwt.helper';
import { UsersRepository } from '../users/user.repository';
import { RefundItemRepository } from '../refund-item/refund-item.repository';
import { StripeModule } from '../orders/stripe/stripe.module';
import { OrderRepository } from '../orders/order/order-repository';

@Module({
    imports: [
        JwtModule.registerAsync({
            useClass: JwtConfigService
        }),
        TypeOrmModule.forFeature([
            PaymentConfigurationRepository,
            PaymentGatewayRepository,
            UsersRepository,
            RefundItemRepository,
            OrderRepository
        ]),
        StripeModule
    ],
    controllers: [PaymentGatewayController],
    providers: [PaymentGatewayService, JwtHelper]
})
export class PaymentGatewayModule {
}
