import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { v4 as uuidv4 } from 'uuid';
import readXlsxFile from 'read-excel-file/node';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { FileUploadsService } from '../../file-uploads/file-uploads.service';
import { BrandRepository } from './brand.repository';
import { CreateBrandDto } from './dto/create-brand.dto';
import { UpdateBrandDto } from './dto/update-brand.dto';
import { Brand } from './entity/brand.entity';

@Injectable()
export class BrandService {
    constructor(
        @InjectRepository(BrandRepository)
        private readonly repository: BrandRepository,
        private readonly fileService: FileUploadsService
    ) {}

    async create(
        dto: CreateBrandDto
    ): Promise<Brand | Error> {
        const res: any = await this.repository.findByBrandName(dto.brandName);
        if (res.length > 0) {
            throw new HttpException(
                "Brand name already exists.",
                HttpStatus.BAD_REQUEST
            );
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Brand created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateBrandDto
    ): Promise<Brand | Error> {
        const brand: any = await this.repository.findOneEntity(id);
        if (!brand) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const data = await this.repository.updateBrand(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Brand updated successfully!", "data", data);
    }

    async findAll(): Promise<Brand[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "Brand data fetched successfully!", "data", data);
    }

    async findOne(id: string): Promise<Brand[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        
        return ResponseUtils.successResponseHandler(200, "Brand data fetched successfully!", "data", data);
    }


    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.buildDeletedData(data, 200, "Brand deleted successfully!");
    }
}
