import { Body, Controller, Delete, Get, Param, Patch, Post } from '@nestjs/common';
import { Constants } from 'src/utils/constants';
import { ColorService } from './color.service';
import { CreateColorDto } from './dto/create-color.dto';
import { UpdateColorDto } from './dto/update-color.dto';

@Controller({
    path: 'colors',
    version: Constants.API_VERSION_1
})
export class ColorController {
    constructor(private readonly colorService: ColorService) {}

    @Post()
    create(@Body() dto: CreateColorDto) {
        return this.colorService.create(dto);
    }

    @Get()
    findAll() {
        return this.colorService.findAll();
    }

    @Get(':id')
    findOne(@Param('id') id: string) {
        return this.colorService.findOne(id);
    }

    @Patch(':id')
    update(@Param('id') id: string, @Body() dto: UpdateColorDto) {
        return this.colorService.update(id, dto);
    }

    @Delete(':id')
    async remove(@Param('id') id: string) {
        return await this.colorService.remove(id);
    }
}
