import { Body, Controller, Delete, Get, Param, Patch, Post, Query, UseGuards } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { ProductReviewService } from "./product-review.service";
import { CreateProductReviewDto } from "./dto/create-product-review.dto";
import { UpdateProductReviewDto } from "./dto/update-product-review.dto";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";

@Controller({ path: "product-review", version: Constants.API_VERSION_1 })

export class ProductReviewController {
    constructor(private readonly service: ProductReviewService) {}

    @Post()
    create(
        @Body() dto: CreateProductReviewDto
    ) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    async findAll(
        @Query('limit') limit: number,
        @Query('page') page: number,
        @Query('productName') productName?: string,
        @Query('rating') rating?: string,
        @Query('date') date?: Date,
        @Query('userName') userName?: string,
    ) {
        return this.service.findAll(limit, page, productName, rating, date, userName);
    }

    @Get(":id")
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    update(@Param('id') id: string, @Body() dto: UpdateProductReviewDto) {
        return this.service.update(id, dto);
    }

    @Delete(":id")
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
