import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateTaxExemptDto} from "./dto/create-tax-exempt.dto";
import {UpdateTaxExemptDto} from "./dto/update-tax-exempt.dto";
import {TaxExempt} from "./entity/tax-exempt.entity";

@EntityRepository(TaxExempt)
export class TaxExemptRepository extends Repository<TaxExempt> {
    async createEntity(dto: CreateTaxExemptDto): Promise<TaxExempt | Error> {
        try {
            const {userId, userName, taxFile} = dto;
            const entity = this.create({
                userName,
                userId,
                taxFile
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<TaxExempt | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateTaxExemptDto): Promise<TaxExempt | Error> {
        try {
            await this.update(id, {
                userName: dto.userName,
                userId: dto.userId,
                taxFile: dto.taxFile
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByUserId(id: string): Promise<TaxExempt[] | Error> {
        try {
            return await this.find({
                where: {
                    userId: id,
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<TaxExempt[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isRejected: false,
                    isApproved: false,
                    isDeleted: false
                },order: { createdAt: "DESC" }
            });
        } catch (e) {
            return e;
        }
    }

    async findApprovedList(query: any): Promise<TaxExempt[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isApproved: true,
                    isRejected: false,
                    isDeleted: false
                },
                order: { createdAt: "DESC" },
            });
        } catch (e) {
            return e;
        }
    }

    async findRejectedList(query: any): Promise<TaxExempt[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isRejected: true,
                    isApproved: false,
                    isDeleted: false
                },
                order: { createdAt: "DESC" },
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(TaxExempt)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}