import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {FileUploadsService} from 'src/modules/file-uploads/file-uploads.service';
import {TaxExemptRepository} from './tax-exempt.repository';
import {CreateTaxExemptDto} from './dto/create-tax-exempt.dto';
import {TaxExempt} from './entity/tax-exempt.entity';
import {UpdateTaxExemptDto} from './dto/update-tax-exempt.dto';
import { DoSpacesService } from '../SpacesModule/SpacesService/doSpacesService';
import { UploadedMulterFileI } from '../SpacesModule/SpacesService';

@Injectable()
export class TaxExemptService {
    constructor(
        @InjectRepository(TaxExemptRepository)
        private readonly repository: TaxExemptRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateTaxExemptDto,
        files: {
            taxFile?: UploadedMulterFileI
        }
    ): Promise<TaxExempt | Error> {
        if (files && files.taxFile) {
            const url = await this.doSpacesService.uploadFile(files.taxFile[0], "uploads/TaxExempt/");
            dto.taxFile = url.toString();
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateTaxExemptDto,
        files: {
            taxFile?: UploadedMulterFileI
        }
    ): Promise<TaxExempt | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (files && files.taxFile) {
            const url = await this.doSpacesService.uploadFile(files.taxFile[0], "uploads/TaxExempt/");
            dto.taxFile = url.toString();
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<TaxExempt[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findApprovedList(): Promise<TaxExempt[] | Error> {
        const data = await this.repository.findApprovedList({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findRejectedList(): Promise<TaxExempt[] | Error> {
        const data = await this.repository.findRejectedList({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<TaxExempt[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findByUserId(id: string): Promise<TaxExempt[] | Error> {
        const data = await this.repository.findByUserId(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }

    async approveUser(id: string): Promise<any | Error> {
        const data = await this.repository.update(id, {isApproved: true});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "User approved successfully!", "data", data);
    }

    async rejectUser(id: string): Promise<any | Error> {
        const data = await this.repository.update(id, {isRejected: true});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "User rejected successfully!", "data", data);
    }
}
